#ifndef _ITR_SILRAW_H_
#define _ITR_SILRAW_H_

#ifdef __cplusplus
extern "C" {
#endif

/* IOL include */
#include "iol.h"

/* for definition of EVENT_HDR, SLB_HDR, etc */
#include "event.h"
#include "etb.h"
#include "tgts_raw.h"

#include "CVStag.ITR_silraw"


/* Define some array dimensions. 
 * These should be the maximum numbers for the final ITR setup.
 * ------------------------------------------------------------ */
#define DIM_NFED          552   /* max number of FEDs                   */
#define DIM_NCHIP         1104  /* max number of read-out chips         */
#define DIM_NCHAMBER      184   /* max number of chambers               */
#define DIM_NCHIPCHAMBER  6     /* max number of chips per chamber      */
#define DIM_NSPARSE       6     /* max number of sparsified data blocks */


/* Now we have to distinguish all kinds of chips, read-out modes
 * and data sizes
 *--------------------------------------------------------------*/ 
/* Chip types */
#define HELIXTYPE       1   /* Helix without DV jitter      */
#define UNKNOWNTYPE     0   /* Unknown chip type            */

/* Chip Counts */
#define NCHIPFEDHELIX   2   /* 2 Helix chips per FED     */

/* Cell Counts */
#define NCELLHELIXMAX   141   /* VB: ITR with different pipeline length ???*/
#define NCELLHELIXSHORT 136   /* VB: helix with short pipeline             */
#define NCELLHELIXLONG  141   /* VB: helix with normal pipeline            */

/* Channel Counts */
#define NDATACHAN     128   /* Channel count of the data              */
#define NCELLCHAN       8   /* Channel count for Helix cell# decoding */
#define NOFFSETCHAN    15   /* Additional Helix channel count per FED */

/* some defines to examine record id's */
#define ITR_SILRAW_FED(i)        ( (i) & RID_NORMAL_FEDRAW )
#define ITR_SILRAW_SPARSE(i)     ( (i) & 0x0001 )
#define ITR_SILRAW_RAW(i)        ( ! ITR_SILRAW_SPARSE(i) )
#define ITR_SILRAW_ERROR(i)      ( (i) & RID_MASK_ERROR )
#define ITR_SILRAW_FED_RAW(i)    ( ITR_SILRAW_FED(i) && ITR_SILRAW_RAW(i)    \
                                             && !ITR_SILRAW_ERROR(i) )
#define ITR_SILRAW_FED_SPARSE(i) ( ITR_SILRAW_FED(i) && ITR_SILRAW_SPARSE(i) \
                                             && !ITR_SILRAW_ERROR(i) )
#define ITR_SILRAW_FED_RAW_ERROR(i)    ( ITR_SILRAW_FED(i) && ITR_SILRAW_RAW(i)    \
                                                   && ITR_SILRAW_ERROR(i) )
#define ITR_SILRAW_FED_SPARSE_ERROR(i) ( ITR_SILRAW_FED(i) && ITR_SILRAW_SPARSE(i) \
                                                   && ITR_SILRAW_ERROR(i) )


/* IOL definitions
   =============== */

typedef struct {
  IOL_MNAME  module[IOL_MLEN];        /* Module name must be first       */
  IOL_SIZE   size;                    /* # of verbs must be second       */ 
  IOL_STRING version;                 /* version of library              */
  IOL_INT    rawdata_type;            /* 0=EVENT, 1=SLB, 2=FED, 3=SLBsub */
  IOL_INT    datavalid_jitter;        /* Data valid jitter On/Off        */
  IOL_INT    datavalid_jitter_offset; /* Latency offset in DV jitter     */
  IOL_INT    datavalid_offset_sl1;    /* Data Valid offset Superlayer1  ms01 */
  IOL_INT    datavalid_offset_sl2;    /* Data Valid offset Superlayer2  ms03 */
  IOL_INT    datavalid_offset_sl3;    /* Data Valid offset Superlayer3  ms05 */
  IOL_INT    datavalid_offset_sl4;    /* Data Valid offset Superlayer4  ms06 */
  IOL_INT    datavalid_offset_sl5;    /* Data Valid offset Superlayer5  ms10 */
  IOL_INT    datavalid_offset_sl6;    /* Data Valid offset Superlayer6  ms11 */
  IOL_INT    datavalid_offset_sl7;    /* Data Valid offset Superlayer7  ms12 */
  IOL_INT    datavalid_offset_sl8;    /* Data Valid offset Superlayer8  ms13 */
  IOL_INT    datavalid_offset_sl9;    /* Data Valid offset Superlayer9  ms14 */
  IOL_INT    datavalid_offset_sl10;   /* Data Valid offset Superlayer10 ms14 */
  IOL_INT    minadcdiff;              /* Min dist between cell # ADC count and average*/
  IOL_INT    dump_header;             /* Dump silraw header              */
  IOL_INT    dump_chips;              /* Dump chip data                  */
  IOL_INT    dump_feds;               /* Dump FED data                   */
  IOL_INT    dump_sparse;             /* Dump sparsified data            */
  IOL_INT    debug_mode;              /* Flag for debugging              */
  IOL_INT    get_faulty_feds;         /* correct/faulty/all (0/1/2) feds  */
  IOL_INT    fed_statistics_period;   /* Period for FED statistics dump  */
  IOL_STRING bad_chip_list;           /* list of chips (not read out)*/
  IOL_INT    cell_check;              /* check cell #'s for consistency  */
  IOL_INT    cell_check_output;       /* cell-check output (0,1,2)       */
  IOL_INT    event_check;             /* perform event check             */
  IOL_INT    repair_mode;             /* try to repair offsets in digits */
} ITR_Silraw_Iol_t;					    
extern ITR_Silraw_Iol_t ITR_silraw_iol;

/* structure definitions
   ===================== 

       
   o FED block structure in HELIX MODE (from 1998 on):
     -------------------------------------------------

     FED Header: 2 byte (short)  Byte Count (length)
                 2 byte (ushort) Record ID
		 2 byte (short)  FLT event #    
		 2 byte (short)  BX #         
		 2 byte (short)  Component ID (0x0200 for ITR)
		 2 byte (short)  FED ID
 
     Data:     284 byte = 2 chips
                           x ( 128 channels + 8 cell# + 6 DV-jitter )
	      (will be 272 byte in 1999 without DV jitter)

     Data Tail:  1 byte          Status & Error Bits
                 1 byte          BX-FLT #
 
     FED Tail:   2 byte (short)  FLT event #

*/

/* structures & union for ITR raw FED data formats
   to account for the different read-out modes */

/* header of silraw event */ 
typedef struct {
  int     major;      /* major number (eg. run #)      */
  int     minor;      /* minor number (eg. event #)    */
  int     evtime;     /* current time                  */
  int     cellshort;  /* pipeline cell number for helix with short pipeline */
  int     celllong;   /* pipeline cell number for helix with long pipeline */
  int     fltevt;     /* FLT event number              */
  int     sltevt;     /* SLT event number              */
  int     bxid;       /* BX number                     */
  int     flag;       /* Flag (to be defined)          */
} ITR_Silraw_Header_t;

/* pointer to FED blocks */
typedef struct {
  short   fedid;      /* FED id                          */
  short   recid;      /* record id                       */
  int     pipelen;    /* pipeline length of the chips    */
  short   superlayer; /* superlayer of this FED          */
  short   length;     /* size of whole FED block         */
  short   headersize; /* length of FED header            */
  short   datasize;   /* length of FED data block        */
  short   tailsize;   /* length of FED tail              */
  short   offset;     /* offset from data to 1. digitiz. */  
  char    *header;    /* pointer to FED / SLB-sub header */
  char    *data;      /* FED data                        */
  char    *datatail;  /* FED data tail                   */
  char    *tail;      /* pointer to FED tail (or NULL)   */
} ITR_Silraw_Fed_t;

/* pointer to read-out chips */ 
typedef struct {
  int     fedid;      /* FED id  (0xABXD, A=crate,B=slot,D=FED) */
  short   recid;      /* record id                              */
  int     chipid;     /* Chip id (0xABCD, C=chip)               */
  int     length;     /* length of all chip data (w/o tail)     */
  int     offset;     /* data starts at *data + offset*chan
                               (necessary for 12-bit data)      */
  int     pipelen;    /* pipeline length                        */
  int     cellnumheight;/* diff. between 0 and 1 of the cellsum   */
  int     flag;       /* to be defined                          */
  char    *data;      /* pointer to chip data                   */
  char    *celldata;  /* pointer to analog cell # data          */
} ITR_Silraw_Chip_t;

/* pointer to possible sparsified event data */
typedef struct {
  int     length;     /* size of pure sparsified data         */
  int     recid;      /* record id of corresponding SLB block */  
  char    *data;      /* pointer to sparsified data           */
} ITR_Silraw_Sparse_t;

/* finally the whole event */ 
typedef struct {
  ITR_Silraw_Header_t header;              /* silraw header               */
  int                 nfed;                /* No of FEDs                  */
  int                 nchip;               /* No of read-out chips        */
  int                 nsparse;             /* No of spars. data blocks    */
  ITR_Silraw_Fed_t    fed[DIM_NFED];       /* FED data                    */
  ITR_Silraw_Chip_t   chip[DIM_NCHIP];     /* chip data                   */
  ITR_Silraw_Sparse_t sparse[DIM_NSPARSE]; /* sparsified data             */
  EVT_DATA_RECORD     *daq;                /* byte swapped event info     */
  ETB_fed             *target_etb;         /* non-byte swapped target slb */
  TGTS_RAW            *target;             /* byte swapped target info    */
} ITR_Silraw_Event_t;


/* function prototypes
   =================== */

/* auxiliary functions */

/* int ITR_silraw_check_event(); */
/* int ITR_silraw_check_offset(unsigned char *data, int offset); */
int ITR_silraw_fit_cell(unsigned char *data);
int ITR_silraw_fit_cell_pipe(unsigned char *data, int pipelen);

int ITR_silraw_read_bad_chip_list();
int ITR_silraw_bad_chip( int chipid );

/* fill ITR_SILRAW_EVENT from raw data (in SLB or FED format)
   and declare it to datpak                               */
int ITR_silraw_get_event( EVENT_HDR *event );
int ITR_silraw_get_CIF_event( SLB_HDR **slb, int nslb, int ext_format );
int ITR_silraw_get_slb( SLB_HDR *slb );
int ITR_silraw_get_slbsub( SLB_SUBHDR *slbsub, int nfed );
int ITR_silraw_get_fed( FED_HDR *fed, int nfed );
int ITR_silraw_etb_get_feds_from_slb( const SLB_HDR *slb, int max,
				  ETB_fed *feds, int* nfed );
short ITR_silraw_get_superlayer(short fedid);
int ITR_silraw_get_datavalidoffset(short superlayer);

int ITR_silraw_fetch_feds_from_etb_list( ETB_fed *etb_fed, int nfed );
int ITR_silraw_fetch_fed_from_etbfed( ETB_fed *etb_fed );
int ITR_silraw_fetch_fed_from_slbsub( SLB_SUBHDR *slb_subhdr );
int ITR_silraw_fetch_fed_from_fed( FED_HDR *fed_hdr );
int ITR_silraw_fetch_chips_from_fed( int ifed, int nchip );

/* fetch existing ITR_silraw event structure */
ITR_Silraw_Event_t* ITR_silraw_fetch_event();

/* fetch cell number from data stream */
int ITR_silraw_fetch_cell_from_data( int *cellshort, int *celllong );

/* reformat raw digitizations to integer */
int ITR_silraw_reformat_data(unsigned char *data, int ndata, int *output );

/* functions for event header access */
int ITR_silraw_get_major     ( int *major );
int ITR_silraw_get_minor     ( int *minor );
int ITR_silraw_get_evtime    ( int *evtime );
  /*int ITR_silraw_get_cell   ( int *cell );*/
int ITR_silraw_get_cellshort ( int *cellshort );
int ITR_silraw_get_celllong  ( int *celllong );
int ITR_silraw_get_fltevt    ( int *fltevt );
int ITR_silraw_get_sltevt    ( int *sltevt );
int ITR_silraw_get_bxid      ( int *bxid );
int ITR_silraw_get_nchip     ( int *nchip );
int ITR_silraw_get_nfed      ( int *nfed );
int ITR_silraw_get_nsparse   ( int *nsparse );
int ITR_silraw_get_nchan     ( int *nchan );

/* functions for read-out chip data access */
int ITR_silraw_get_chipid      ( int chip,   int *chipid );
int ITR_silraw_get_pipelen     ( int chip,   int *pipelen);
int ITR_silraw_get_data_chip   ( int chip,   char **data );
int ITR_silraw_get_data_chipid ( int chipid, char **data );

/* functions for FED block access */
int ITR_silraw_get_fedid    ( ITR_Silraw_Fed_t *fed, int *fedid );

/* DAQ information */
EVT_DATA *ITR_silraw_get_daqdata();


/* routines for helix data debugging */
int ITR_silraw_check_event();
int ITR_silraw_check_cell_consistency();
int ITR_silraw_check_offset(unsigned char *data, int offset);
int ITR_silraw_fit_cell(unsigned char *data);

/* dump data to standard output */
int ITR_silraw_dump_event   ();
int ITR_silraw_dump_chips   ();
int ITR_silraw_dump_feds    ();
int ITR_silraw_dump_header  ();
int ITR_silraw_dump_rawdata ( char *slbdata );

/* program entry points */
int ITR_silraw_param_init();
int ITR_silraw_startprogram();
int ITR_silraw_event( char *rawdata, int nfed );
int ITR_silraw_endrun();


#ifdef __cplusplus
}
#endif

#endif
