#include <string.h>
#include <stdio.h>
#include "aframe.h"
#include "iol.h"
/**************************************************/
/* Example header file for a user with module     */
/**************************************************/

/* The preferences consist of some list of values that
   can be set by verbs and some values or pointers
   that cannot be modified (such as pointers to
   data structures ).  Here, the definiton of the
   "Rest" of the structure should be entered.  The
   Example is rather simple: just a character array.
   The typedefs used should start with some indicator
   of the module name (so that typedefs are not stepped
   on between modules) */

typedef char  USER_REST;
#define USER_REST_ARRAY_SIZE_1 100

/* The structure for the module is now defined.  It
   must start with a module name, the number of
   modifiable entries and then the entries themselves. 
   After that, the remainder of the structure can
   be anything. */

struct daqin_iols{
 IOL_MNAME   module[IOL_MLEN];       /* The module name must be first      */
 IOL_SIZE    size            ;       /* The number of verbs must be second */
 IOL_STRING  version         ;       /* The verbs are then listed          */
 IOL_STRING  consumer        ;
 IOL_DOUBLE  twice           ; 
 IOL_FLOAT   fraction        ; 
 IOL_FLAG    flag            ; 
 IOL_INT     shoesize        ; 
 IOL_HEX     ahexval         ; 
 USER_REST    junk[USER_REST_ARRAY_SIZE_1]    ;       /* The rest of the structure can be anything */
} typedef DAQIN_IOLS         ;

/* When there is a "rest" of the structure, the value should be
   computed and stored in a #defined constant. This is useful 
   for the stucture initialization.  As an example, this
   is easily computed by dividing the size of the array
   by its sizeof for the type it is. */

#define USER_REST_SIZE ((USER_REST_ARRAY_SIZE_1)*(sizeof(USER_REST)))

/* The structure can then be initialized here.  This is
   done by simply filling in the name of the structure.  The
   number of verbs gets computed from the NDAQ_IOL function
   when given the size of the non-preference accessible structure.
   The initialization is done by giving the verb name, the
   initial value, and a string describing its type -- or better,
   the defined types.  Note that if anything other than
   a string type is used, then a null string must be inserted to
   pad out the substructure */

#define DAQIN_NVERBS IOL_NVERBS(DAQIN_IOLS,USER_REST_SIZE)

static DAQIN_IOLS daqin_simple={ "daqin2", DAQIN_NVERBS,
                                {"version"      , "1.00.00" ,        IOL_TYPE_STRING},
                                {"consumer name", "silcon\0",        IOL_TYPE_STRING},
                                {"double frac"  ,  0.22     , " ",   IOL_TYPE_DOUBLE},
                                {"fraction left",  0.1      , " ",   IOL_TYPE_FLOAT },
                                {"flag for you" ,  1        , " ",   IOL_TYPE_FLAG  },
                                {"shoesize"     ,  47       , " ",   IOL_TYPE_INT   },
                                {"hex for you"  ,  44       , " ",   IOL_TYPE_HEX   } };
DAQIN_IOLS daqin_simple_defaults;
int main(int argc, const char **argv)
{
  float  ftest;
  int    flagi;
  double dfred;
  int    ifred;
  int    ioff;
  int    ioff_value;
  int    ihexi;
  char   valueload[132];
  int i;
  iol_init( "dummy", 0 );
  memcpy(&daqin_simple_defaults,&daqin_simple,IOL_BYTESIZE(daqin_simple.size));
  iol_dumpstruct((IOL_GENERIC *)&daqin_simple,sizeof(DAQIN_IOLS));
  printf("sizeof(IOL_STRING)=%d sizeof(IOL_VSTRING)=%d         \n",sizeof(IOL_STRING),sizeof(IOL_VSTRING)      );
  printf("sizeof(IOL_INT)   =%d sizeof(IOL_VINT)   =%d DLENI=%d\n",sizeof(IOL_INT)   ,sizeof(IOL_VINT)   ,IOL_DLENI);
  printf("sizeof(IOL_FLOAT) =%d sizeof(IOL_VFLOAT) =%d DLENF=%d\n",sizeof(IOL_FLOAT) ,sizeof(IOL_VFLOAT) ,IOL_DLENF);
  printf("sizeof(IOL_DOUBLE)=%d sizeof(IOL_VDOUBLE)=%d DLEND=%d\n",sizeof(IOL_DOUBLE),sizeof(IOL_VDOUBLE),IOL_DLEND);
  printf("sizeof(IOL_HEX)   =%d sizeof(IOL_VHEX)   =%d DLENH=%d\n",sizeof(IOL_HEX)   ,sizeof(IOL_VHEX)   ,IOL_DLENH);
  printf("size=%d\n",daqin_simple.size);
  printf("Pointers: \n");
  for(i=0;i<daqin_simple.size;++i)
    {
      ioff=IOL_OFFSET+i*sizeof(IOL_STRING);
      ioff_value=ioff+IOL_VOFF;
      printf("offsets: i=%d ioff=%d ioff_value=%d\n",i,ioff,ioff_value);
    }
  i=3;
  printf("hexi: %s %x %s\n",daqin_simple.ahexval.name,
	 daqin_simple.ahexval.value,daqin_simple.ahexval.type);
  printf("hex verb: %s\n",IOL_GET_VERB(daqin_simple,i));
  printf("hex type: %s\n",IOL_GET_TYPE(daqin_simple,i));
/* dump the values in the structure */
  printf("intial values: \n");
  iol_dumpval(stdout,(IOL_GENERIC *)&daqin_simple,(IOL_GENERIC *)&daqin_simple_defaults,
	      strlen(daqin_simple.module));
  daqin_simple.shoesize.value=88;
  printf("values after direct access: \n");
  iol_dumpval(stdout,(IOL_GENERIC *)&daqin_simple,(IOL_GENERIC *)&daqin_simple_defaults,
	      strlen(daqin_simple.module));

/* Access the stucture with pointer arithmetic 
   and modify a value */
  i=5;
  ioff=IOL_OFFSET+i*sizeof(IOL_STRING);
  ioff_value=ioff+IOL_VOFF;
  printf("ioff=%d ioff_value=%d\n",ioff,ioff_value);
  *((int*)(daqin_simple.module+ioff_value)) = 44;
  printf("values after pointer access\n");
  iol_dumpval(stdout,(IOL_GENERIC *)&daqin_simple,(IOL_GENERIC *)&daqin_simple_defaults,
	      strlen(daqin_simple.module));

/* use the defined inline functions to modify an 
   integer value with pointer arithmetic */
  IOL_LOC_VAL_I(daqin_simple,5)=97.33;
  printf("values after define pointer access to i\n");
  iol_dumpval(stdout,(IOL_GENERIC *)&daqin_simple,(IOL_GENERIC *)&daqin_simple_defaults,
	      strlen(daqin_simple.module));

/* use the defined inline functions to modify a
   real value with pointer arithmetic    */
  IOL_LOC_VAL_F(daqin_simple,3)=0.883;
  printf("values after define pointer access to f\n");
  iol_dumpval(stdout,(IOL_GENERIC *)&daqin_simple,(IOL_GENERIC *)&daqin_simple_defaults,
	      strlen(daqin_simple.module));

/* use the defined inline function to modify a
  string value with pointer arithmetic */
  strcpy(IOL_LOC_VAL_S(daqin_simple,1),"duhconsumer");
  printf("values after define pointer access to s\n");
  iol_dumpval(stdout,(IOL_GENERIC *)&daqin_simple,(IOL_GENERIC *)&daqin_simple_defaults,
	      strlen(daqin_simple.module));

/* load a value with a subroutine */
  ifred=42;
  sprintf(valueload,"%d",ifred);
  iol_load_value((IOL_GENERIC*)&daqin_simple,5,valueload);
  printf("values after define pointer access to i with load value\n");
  iol_dumpval(stdout,(IOL_GENERIC *)&daqin_simple,(IOL_GENERIC *)&daqin_simple_defaults,
	      strlen(daqin_simple.module));

/* load a value with a subroutine */
  dfred=0.1142;
  sprintf(valueload,"%lf",dfred);
  iol_load_value((IOL_GENERIC*)&daqin_simple,2,valueload);
  printf("values after define pointer access to double with load value\n");
  iol_dumpval(stdout,(IOL_GENERIC *)&daqin_simple,(IOL_GENERIC *)&daqin_simple_defaults,
	      strlen(daqin_simple.module));

/* load a hex value with a subroutine */
  ihexi=0xDEAD;
  sprintf(valueload,"%x",ihexi);
  iol_load_value((IOL_GENERIC *)&daqin_simple,6,valueload);
  printf("values after define pointer access to h with load value\n");
  iol_dumpval(stdout,(IOL_GENERIC *)&daqin_simple,(IOL_GENERIC *)&daqin_simple_defaults,
	      strlen(daqin_simple.module));

/* load a flag value with a subroutine */
  flagi=-3;
  sprintf(valueload,"%d",flagi);
  iol_load_value((IOL_GENERIC *)&daqin_simple,4,valueload);
  printf("values after define pointer access to flag with load value\n");
  iol_dumpval(stdout,(IOL_GENERIC *)&daqin_simple,(IOL_GENERIC *)&daqin_simple_defaults,
	      strlen(daqin_simple.module));

  return 1;

}

