/*----------------------------------------------------------------- 
-  History:                                                       -  
-  ========                                                       -
-                                                                 - 
-  VB Aug 1999: new cluster-header                                -
-  VB Sep 1999: new histo-ID convention                           - 
-  VB Sep 1999: new iol definition full_histo                     -
-  VB Oct 1999: fed-data added                                    - 
-  VB Nov 1999: BXspectra for every chamber                       -
-  VB Dec 1999: paw histos added                                  - 
-  WG Feb 2000: trim list of histograms for a 'shift proof' version
-------------------------------------------------------------------*/

/*this version: corresponding to marvin3-6 with the "big" cluster-structure*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "pub.h" 
#include "rhp.h" 
#include "etb.h" 
#include "grr.h" 
#include "component.h" /*for COMP_N...*/

#include "aframelib.h"
#include "ITR_silraw.h" 
#include "itrleda.h" 
#include "ITR_marvin.h" 
#include "itrrhp.h" 

extern char *ITR_leda_getChamberDescr(int);

#include "hbook.h"
#include "higz.h"
#define NSPACE 36000000
#include "pawc.h"


#define NCHAMBER 184
#define MINBX    180
#define MAXBX    220
#define NEVMON   200

/* IOL definitions */
ITR_rhp_Iol_t ITR_rhp_iol={
  xcvsMODULE_itrrhp,
  IOL_NVERBS(ITR_rhp_Iol_t,0),
  {"version"                , xcvsVERSION_itrrhp, IOL_TYPE_STRING},
  {"use itrrhp"             , ITRRHP_ON, " ",     IOL_TYPE_INT},
  {"update period"          , 10,  " ",           IOL_TYPE_INT},
  {"full histo"             , 1,   " ",           IOL_TYPE_INT},
  {"whichchamber"           , 0x1000," ",         IOL_TYPE_INT},
  {"debug mode"             , ITRRHP_OFF, " ",    IOL_TYPE_INT},
  {"histevt store"          , 2000,       " ",    IOL_TYPE_INT},
  {"store text"             , 1000,       " ",    IOL_TYPE_INT},
  {"text switch"            , 0,          " ",    IOL_TYPE_INT},
  {"update binsn"           , 1000,       " ",    IOL_TYPE_INT},
  {"path histout"           , "none",             IOL_TYPE_STRING},
};							    


/* Flag that param_init was done */
static int paraminit_done = 0;

/* Arrays for BX contribution plot */
static float bxevent[NBUCKET];   /* event count per bucket   */ 
static float bxclusraw[NBUCKET]; /* raw  cluster count per bucket */ 
static float bxclus[NBUCKET];    /* good cluster count per bucket */ 
static float bxclevraw[NBUCKET]; /* raw  cluster/event per bucket */ 
static float bxclev[NBUCKET];    /* good cluster/event per bucket */ 

/* Array for Clus/Ev versus Time plot */
static float clusev[NTIMEBIN];     /* event count per bucket   */ 
static float clusevraw[NTIMEBIN];  /* event count per bucket   */ 

static unsigned int recolevel = 1; /* 1 == raw */

static ITR_rhp_Chambernumber_t chamberassignment[NCHAMBER];
static ITR_rhp_BX_t            bxspectrum[NCHAMBER];
       ITR_rhp_ncl_t           my_nclarr[NCHAMBER];
   
static int event = 0;

/* formula for ID */
unsigned int DQ_ID( int subdid, int recid, int histid ) {
  return  ( ( subdid<<28 ) | ( recid<<24 ) | histid );
}


/*-----------------------------------------------------------------------------
 *
 * Initialize itrrhp_iol
 *
 *---------------------------------------------------------------------------*/
int ITR_rhp_param_init()
{
  char *routine = "ITR_rhp_param_init";

  /* Check if we haven't called it yet */
  if ( paraminit_done ) {
    aframe_mess(routine,GRR_SWARNING,
		"ITR_rhp_param_init already called, skipping...");
    return GRR_OK;
  }
  paraminit_done = 1;

  /* tell the user that its alive */
  aframe_mess(routine,GRR_SINFO,"ITR_rhp version %s started.",xcvsVERSION_itrrhp);

  /* declare ITR_rhp_iol to IOL */
  iol_declare_structure( (IOL_GENERIC *) &ITR_rhp_iol ); 

  return GRR_OK;
}


/*-----------------------------------------------------------
 *
 * Initialize online monitoring with rhp
 *
 *----------------------------------------------------------*/
int ITR_rhp_init()
{
#define CHARGELOW     0
#define CHARGEHIGH  300
#define NCHARGE     300

#define NSN         200

#define NSTRIP      768 /*VB: ITR*/

  int  i,ii,chamberloop=0,chipnumber;
  FILE *chamberMapFile, *chipMapFile;
  int  ichamber, ichip;
  char *chambername;
  char title[100];

  char *routine = "ITR_rhp_init";

  LedaTableIterator( ITR_Chamber, chamberIt );
  /*  LedaTableIterator( ITR_Chip,    chipIt    );*/

  if ( !ITR_rhp_iol.use_rhp.value ) {
    aframe_mess( routine, GRR_SINFO,"ITR_RHP will not be used.");
    return GRR_OK;
  }
  
  /*VB: booking histograms for file*/
  hlimit(NSPACE);
  iginit(0);


  /* initializing rhp */
  pub_sinit();
  pub_sbind();
  rhp_msinit();
  rhp_msbind();


  /* book general histograms */
  hbook1( 11, "Cluster/Event versus BX number",NBUCKET, -0.5, (float) NBUCKET-0.5);
  hbook1( 21, "Cluster/Event", 100, 0., 100. );

  /* these are important for the shift crew as well */
  rhp_hbook1 (DQ_ID(COMP_N_ITR, recolevel, 11), 
	      "Cluster per Event vs. BX number", 
	      NBUCKET, -0.5, (float) NBUCKET-0.5);

  chamberMapFile = fopen ("chamberMap.txt", "w");
  if (chamberMapFile == NULL) 
      perror ("while opening chamberMap.txt");

  chipMapFile = fopen ("chipMap.txt", "w");
  if (chipMapFile == NULL) 
    perror ("while writing chipMap.txt");

  ichamber = 0;
  ichip    = 0;

  /* book chamber-dependent histograms by looping over Leda chambers */
  for( chamberIt.begin(); chamberIt.valid(); chamberIt.next() ) {
    LedaTableRow(ITR_Chamber) *chamber = chamberIt.get();
    LedaRelationIterator(chamber->chip, chipIt);

    if (chamberMapFile)
      fprintf (chamberMapFile, "%d   %d\n", ichamber, chamber->id);

    ichamber ++;
    
    if (chamber->status){
      chipnumber=1;
      chambername = ITR_leda_getChamberDescr(chamber->id);

      printf ("%s\n", ITR_leda_getChamberDescr(chamber->id));

      /* charge */
      sprintf(title, "Cluster Charge %s", chambername);
      hbook1(600000+chamber->id,title,NCHARGE, CHARGELOW, CHARGEHIGH);
  
      /* signal/noise */
      sprintf(title, "S/N %s", chambername);
      hbook1( 200000+chamber->id, title, NSN, 0, 100 );
      
      /* occupancy */
      sprintf(title, "Wiremap %s", chambername);
      hbook1( 80000+chamber->id, title, NSTRIP, 0., (float) NSTRIP );
         
      /* occupancy */
      sprintf(title, "Occ %s", chambername);
      hbook1( 90000+chamber->id, title, NSTRIP, 0., (float) NSTRIP );
      rhp_hbook1 ( DQ_ID(COMP_N_ITR, recolevel, 90000+chamber->id), 
		   title, NSTRIP, 0., (float) NSTRIP );

      /* clustersize */ 
      sprintf(title, "Clustersize %s", chambername);
      hbook1(40000+chamber->id, title, 20, -0.5, 19.5 );
    
      /* max pulseheight */
      sprintf(title, "Max PH %s", chambername);
      hbook1(30000+chamber->id, title, 100, -0.5, 95.5);
      
      /* cl/ev */
      sprintf(title, "Cluster/Event %s", chambername);
      hbook1(50000+chamber->id, title, 100, -0.5, 95.5);

      /*noise = dvar*/
      sprintf(title, "Subtr Noise Ch %s", chambername);
      hbook1(800000+chamber->id, title, NSTRIP, 0., (float) NSTRIP);
         
      /*BXspectrum*/
      sprintf(title, "BXspectrum %s", chambername);
      hbook1(150000+chamber->id, title, NBUCKET, -0.5, (float) NBUCKET-0.5);
      
      sprintf(title, "Detail Ch %s", chambername);
      hbook1(160000+chamber->id,title, MAXBX-MINBX, MINBX - 0.5,MAXBX - 0.5);
      
      sprintf(title, "BXspectrum Ch %s", chambername);
      hbook1(170000+chamber->id, title, NBUCKET, -0.5, (float) NBUCKET-0.5);
      


      for ( chipIt.begin(); chipIt.valid(); chipIt.next() ) {
	LedaTableRow(ITR_Chip)     *ledachip     = chipIt.get();
	if (ledachip->status) {
	  sprintf(title, "Proj. Chip %x-%d",chamber->id,chipnumber);
	  hbook1(900000+ledachip->id,title,100,-10.,10.);
	  sprintf(title, "colnr Chip %x-%d",chamber->id,chipnumber);
	  hbook1(950000+ledachip->id,title,100,0.,100.);
	  fprintf (chipMapFile, "%4d\t%5d\n", ichip, ledachip->id);
	  ichip ++;
	}
	chipnumber++;
      }
    }/*end of if-chamber->status*/
  }/*end of loop over chambers*/ 
  
  if (chamberMapFile)
    fclose (chamberMapFile);
  
  if (chipMapFile) {
    fflush (chipMapFile);
    fclose (chipMapFile);
  }

  /* book chip-dependent histograms by looping over Leda chips */
  /*---wg--- for( chipIt.begin(); chipIt.valid(); chipIt.next() ) { 
    LedaTableRow(ITR_Chip) *chip = chipIt.get();  
    if (chip->status) sprintf(title, "Proj. Chip %x", chip->id);  
    hbook1(900000+chip->id, title,100,-10.,10.); 
  }---wg---*/
  /*end of loop over chips*/

  /* reset static arrays */
  for ( i=0; i<NBUCKET; i++ )
    bxevent[i] = bxclus[i] = bxclev[i] = bxclusraw[i] = bxclevraw[i] = 0.;
  for ( i=0; i<NTIMEBIN; i++ )
    clusev[i] = 0.;
  for (i=0;i<NCHAMBER;i++){
    chamberassignment[i].ID           =0;
    chamberassignment[i].midmaxheight =0;
    chamberassignment[i].ncluster     =0;
    bxspectrum[i].ID = 0;
    for ( ii=0; ii<NBUCKET; ii++ ){
      bxspectrum[i].bxevent[ii]=0;
      bxspectrum[i].bxclus[ii] =0;
      bxspectrum[i].bxclev[ii] =0;
      my_nclarr[i].ID =0;
    }
  } 
  for( chamberIt.begin(); chamberIt.valid(); chamberIt.next() ) {
    LedaTableRow(ITR_Chamber) *chamber = chamberIt.get();
    if (chamber->status){
      bxspectrum[chamberloop].ID       = chamber->id;
      my_nclarr[chamberloop].ID        = chamber->id;
      chamberassignment[chamberloop].ID= chamber->id; 
      chamberloop++;
    }
  }

  aframe_mess( routine, GRR_SINFO,"RHP initialized and histograms booked.");

  return GRR_OK;
}


/*------------------------------------------------------------
 *
 * Monitor one event with rhp
 *
 *-----------------------------------------------------------*/
int ITR_rhp_event( EVENT_HDR *evt_hdr )
{
 
  ITR_Silraw_Event_t   *silraw;
  ITR_Marvin_Sevt_t    *sevt;
  ITR_Marvin_Header_t  *sevtheader;
  ITR_Marvin_Cluster_t *cluster;
  
  static int idhist[MAXHIST];
  static int  nhist, i, rc;

  static int nclus    = 0;
  static int nclusraw = 0;

  int           my_ncl;
 
  /* variables for DAQ SLB block */
  EVT_DATA_RECORD daq;
  int             ndaq = 0;
  int             physBX;

  int                  countme,zaehler,stripcnt,ii,iii,jj,jjj,feddata;
  int                  mchipid, strip=0, rawdata=0,silchip; 
  ITR_Marvin_Event_t   *mevent;
  ITR_Marvin_Chamber_t *mchamber;
  ITR_Marvin_Chip_t    *mchip;
  float                pulsemax;
  char                 histfile[256];

  LedaTableIterator( ITR_Chamber, chamberIt );
 
  char *routine = "ITR_rhp_event";
 
  if ( !ITR_rhp_iol.use_rhp.value )
    return GRR_OK;

  if ( ITR_rhp_iol.debug_mode.value )
    aframe_mess(routine,GRR_SINFO,"Entering RHP event...");

  
  /* get silraw event */
  silraw = ITR_silraw_fetch_event();
  if ( !silraw ) aframe_mess( routine, GRR_SERROR,"no ITR_Silraw_event_t found");

  /* get sparsified event and # of clusters */
  sevt = ITR_marvin_fetch_sevt();
  if ( sevt == NULL ) { aframe_mess(routine,GRR_SERROR,
				    "NULL pointer to sparsified information!");
          return GRR_FISHY;
  }
  sevtheader = ITR_marvin_sevtHeader( sevt->data );  
  if ( ITR_rhp_iol.debug_mode.value )
    ITR_marvin_dumpClusters( sevt );

  mevent   = ITR_marvin_fetch_event();
  if ( !mevent ) aframe_mess(routine,GRR_SERROR,"no ITR_Marvin_Event_t found!");

  if (event == 0){
    sprintf(histfile, "%s/dmon_itr_%d.his",
	    ITR_rhp_iol.path_histout.value,silraw->header.major);
    /*hropen(0,"hist", histfile,"",0,);*/
  }

  /*=================CLUSTER HISTOGRAMS=======================*/ 
  /* loop over clusters to fill histograms */
  cluster = ITR_marvin_sevtCluster( (char*) sevt->data, -1 );
  my_ncl  = 0;
  for (jj=0;jj<NCHAMBER;jj++) my_nclarr[jj].ncl=0;
  while ( cluster ) {

    /* get Leda chamber id from database */
    LedaTableRow(ITR_Chamber) *chamber = ITR_Chamber->get->key( cluster->chamber );
    if ( !chamber ) {
      aframe_mess(routine,GRR_SERROR,
		  "ITR chamber %d not in database", cluster->chamber );
      cluster = ITR_marvin_sevtCluster(NULL, -1);
      continue;
    }
    if (!chamber->status){
      /*  aframe_mess(routine, GRR_SERROR, */
      /*              "ITR chamber %d is flagged bad", cluster->chamber ); */
      cluster = ITR_marvin_sevtCluster(NULL, -1);
      continue;
    }

    /* fill cluster histograms */
    if (cluster->charge) hf1(600000+chamber->id,cluster->charge,1.);
    hf1(200000+chamber->id,cluster->sn,    1.);
    hf1( 80000+chamber->id,cluster->cog,   1.);
    rhp_hf1 (DQ_ID(COMP_N_ITR, recolevel, 90000+chamber->id), cluster->cog, 1.);

    /*     hf1( 90000+chamber->id,cluster->cog,   1.); */
    if (cluster->width)  hf1(40000+chamber->id, cluster->width, 1.);

    /* calculation of pulsemax */
    pulsemax=0;
    for (jjj=0;jjj<=(cluster->end-cluster->first);jjj++){
      if (-cluster->digits[jjj]-cluster->localmean > pulsemax)
	pulsemax = -(cluster->digits[jjj])-(cluster->localmean);
    }
      
    /*fill time-mid from max.pulseheight*/
    if (event > NEVMON)
      for (i=0;i<NCHAMBER;i++){
	if (chamber->id == chamberassignment[i].ID){
	  /*chamberassignment[i].midmaxheight += pulsemax;*/
	  chamberassignment[i].ncluster     += 1;
	}
      }
    if (pulsemax>0) hf1(30000+chamber->id, pulsemax, 1.);
    

    /*VB: hier sollten noch Cuts fuer Cluster rein!! und dann gecuttete cog etc!*/ 
    my_ncl++;
    for (i=0;i<NCHAMBER;i++)
      if (my_nclarr[i].ID == chamber->id){
	(my_nclarr[i].ncl) += 1;
      }

    
    /*for (i=0;i<NCHAMBER;i++) 
      if (event%200==0 && chamberassignment[i].ID==chamber->id){ 
      printf("ID 0x%4x, ncluster %4d, nevents %5d, clu/ev %f\n", 
      chamberassignment[i].ID,chamberassignment[i].ncluster,event,
      (float)chamberassignment[i].ncluster/(event-199)); 
      if (chamberassignment[i].ncluster) 
      printf(" midmaxheight %2.2f\n\n", 
      chamberassignment[i].midmaxheight/chamberassignment[i].ncluster); 
      }  */
    
    /* get next cluster */
    cluster = ITR_marvin_sevtCluster(NULL, -1);
  } /*end of loop while(cluster)*/
  


  /*=================DAQ HISTOGRAMS=======================*/ 
  /* get bucket ID and count events and clusters per event */
  rc = ETB_get_daqs( evt_hdr, 1, &daq, &ndaq);
  if ( rc != ETB_ok || ndaq != 1 ) {
    aframe_mess(routine,GRR_SERROR,
		"Could not access DAQ event record, number of returned blocks = %d",
		ndaq);
    return GRR_FISHY;
  }

  physBX = (int) daq.data.physBX;
  if ( physBX >= 0 && physBX < NBUCKET ) {
    bxevent[physBX]++;
    bxclusraw[physBX] += sevtheader->ncl;
    bxclus[physBX]    += my_ncl;
    if ( bxevent[physBX] > 0 ) {
      bxclevraw[physBX] = bxclusraw[physBX] / bxevent[physBX];
      bxclev[physBX]    = bxclus[physBX]    / bxevent[physBX];
    }
    for( chamberIt.begin(); chamberIt.valid(); chamberIt.next() ) {
      LedaTableRow(ITR_Chamber) *chamber = chamberIt.get();
      if (chamber->status){
	for (ii=0;ii<NCHAMBER;ii++)
	  if (bxspectrum[ii].ID == chamber->id){
	    ((bxspectrum[ii].bxevent)[physBX]) += 1;
	    (bxspectrum[ii].bxclus)[physBX] += my_nclarr[ii].ncl;
	    if ( (bxspectrum[ii].bxevent)[physBX] > 0 ) {
	      (bxspectrum[ii].bxclev)[physBX] = 
		(bxspectrum[ii].bxclus)[physBX] / (bxspectrum[ii].bxevent)[physBX];
	    }
	  }
      }
    }
  }
  else
    aframe_mess(routine,GRR_SERROR,
		"PhysBX from DAQ record out of range: %d ", physBX);

  /* add up clusters for time-history plot */
  nclusraw += sevtheader->ncl;
  nclus    += my_ncl;
    
   /*=================MARVIN HISTOGRAMS=======================*/ 
  /*====for chambers===*/
  for( chamberIt.begin(),ii=0; chamberIt.valid(),(ii>=0&&ii<mevent->nchamber); 
       chamberIt.next(),ii++ ) {
    LedaTableRow(ITR_Chamber) *chamber = chamberIt.get();
    if (chamber->status){
      mchamber = mevent -> chamber[ii]; 
      if (!mchamber){
	if (ITR_rhp_iol.debug_mode.value)
	  aframe_mess(routine, GRR_SINFO, "can't find leda chamber %x",chamber->id);
	continue;
      }
      if (mchamber->first > mchamber->last){
	if (ITR_rhp_iol.debug_mode.value)
	  aframe_mess(routine, GRR_SINFO, "chamber %x not ok",mchamber->chamberid);
	continue;
      }
      if (mchamber->chamberid != chamber->id){
	if (event ==0)
	  aframe_mess(routine, GRR_SERROR, 
		      "Found marvin chamber %x, expected leda chamber %x",
		      mchamber->chamberid, chamber->id);
	ii--; /*VB "--" because this ii should have the same value after continue!!!!*/
	continue;
      }
      /*else printf("VB:marvin == leda chamber %d\n",chamber->id);*/

      /*marvin chamber data and dvar*/
    

      /*  if (ITR_rhp_iol.histevt_store.value < event){ */
      /*  hreset(100000+mchamber->chamberid, ""); 
	  rhp_reset(DQ_ID(COMP_N_ITR,recolevel,100000+mchamber->chamberid));*/
      /*  hreset(800000+mchamber->chamberid, ""); */ 

      /*
      for(stripcnt=mchamber->first;stripcnt<=mchamber->last;stripcnt++){
	hf1(100000+mchamber->chamberid,stripcnt,(float)(*(mchamber->data[stripcnt])));
	rhp_hf1(DQ_ID(COMP_N_ITR,recolevel,100000+mchamber->chamberid),
		stripcnt,(float)(*(mchamber->data[stripcnt])));
	 hf1(800000+mchamber->chamberid,stripcnt,(float)(*(mchamber->dvar[stripcnt]))); 
      }
      */  

      /* clu/evt */
      if (event > NEVMON)
	for (i=0;i<NCHAMBER;i++) 
	  if (chamberassignment[i].ID==mchamber->chamberid){ 
	    hf1(50000+mchamber->chamberid,(float)chamberassignment[i].ncluster,1.);
	    chamberassignment[i].ncluster = 0;
	  } 
     
      /*  } */
  
      /*===for chips===*/
      /*marvin chip unproc. data and pedestals*/
      if (event > ITR_rhp_iol.histevt_store.value){
	for (zaehler=0;zaehler<mchamber->nchip;zaehler++){
	  mchip=mchamber->chip[zaehler];
	  mchipid = mchip->chipid;
	  if (mchip->chipstatus < 0 || !mchip){
	    if (ITR_rhp_iol.debug_mode.value)
	      aframe_mess(routine, GRR_SERROR, "chip %x not ok",mchipid);
	    continue;
	  }
	  for (silchip = 0;silchip<silraw->nchip;silchip++){
	    if (silraw->chip[silchip].chipid == mchipid)
	      hf1(950000+mchipid,(float)silraw->chip[silchip].cellnumheight,1.);
	  }
	 
	  /*	  if (zaehler==0){
	    rhp_reset(DQ_ID(COMP_N_ITR,recolevel,400000+mchamber->chamberid));
	    rhp_reset(DQ_ID(COMP_N_ITR,recolevel,300000+mchamber->chamberid));
	     hreset(300000+mchamber->chamberid,"");  
	       hreset(400000+mchamber->chamberid,"");
	  }*/
	  
	  /*fill histograms for whole chamber*/
	  /* 	if (ITR_rhp_iol.histevt_store.value < event){ */
	  for(countme=0;countme<128;countme++){
	    strip = zaehler*128 + countme;
	    /*pedestals*/
	    /* hf1(300000+mchamber->chamberid,strip,(float)mchip->peds[countme]); */
	    /*  rhp_hf1(DQ_ID(COMP_N_ITR,recolevel,300000+mchamber->chamberid),
		    strip,(float)mchip->peds[countme]);*/
	    if (strip >= mchamber->first && strip <= mchamber->last){
	      /*marvin raw data*/
	      rawdata = (int) mchip->digi[countme]; 
	      /*hf1(400000+mchamber->chamberid,strip,(float)rawdata);*/ 
	      /* rhp_hf1(DQ_ID(COMP_N_ITR,recolevel,400000+mchamber->chamberid),
		      strip,(float)rawdata);*/
	      /*fill histograms for chip*/
	      hf1(900000+mchipid, (float)mchip->data[countme], 1);
	      
	    }
	  }
	}/*end of loop over chips*/
      
	/*===and for feds===*/

	/*silraw fed data*/

	/*for (zaehler=0;zaehler<silraw->nfed;zaehler++){
	  hreset(500000+mchamber->chamberid,"");
	  for (countme=0;countme<280;countme++){
	    feddata = (int)silraw->fed[zaehler].data[countme];
	    hf1(500000+mchamber->chamberid, countme,(float)feddata);
	  }
	}*/ /*end of loop over feds*/
     }
    }
  }/*end of loop over chambers*/


  /*=================GENERAL HISTOGRAMS=======================*/ 
  /* fill general histograms */
  hf1( 20, (float) sevtheader->ncl, 1.);
  hf1( 21, (float) my_ncl, 1.); 
  
  /*=================PUBLISHING=======================*/ 
  /* publish them every now and then */
  if ( event%ITR_rhp_iol.update_period.value == 0 ) {
    
    /* fill bunch contribution plot only here to save time */
    hpak     (11,bxclev);
    rhp_hpak (DQ_ID(COMP_N_ITR, recolevel, 1), bxclev);
  
    /*fill BXspectrum for each chamber*/
    for( chamberIt.begin(),ii=0; chamberIt.valid(); chamberIt.next(),ii++ ) {
      LedaTableRow(ITR_Chamber) *chamber = chamberIt.get(); 
      if (chamber->status) 
	for (ii=0;ii<NCHAMBER;ii++) 
	  if (chamber->id == bxspectrum[ii].ID){ 
	    hpak ( 150000+chamber->id, bxspectrum[ii].bxclev); 
	    hpak ( 170000+chamber->id, bxspectrum[ii].bxclus); 
	    hreset(160000+chamber->id,""); 
	    for (i=MINBX;i<MAXBX;i++) 
	      hf1(160000+chamber->id,i,(bxspectrum[ii].bxclev)[i]); 
	  } 
    } 
    
    /* fill time-history plot of clus/event */
    if ( event > 0 ) {
      for ( i=1; i<NTIMEBIN; i++ ) {
	clusev[i-1]    = clusev[i];
	clusevraw[i-1] = clusevraw[i];
      }
      clusev[NTIMEBIN-1]    =  (float) nclus / (float) ITR_rhp_iol.update_period.value;
      clusevraw[NTIMEBIN-1] =  (float) nclusraw	/ (float) ITR_rhp_iol.update_period.value;
      nclus = nclusraw = 0;
    }

    for( chamberIt.begin(); chamberIt.valid(); chamberIt.next()) {
      LedaTableRow(ITR_Chamber) *chamber = chamberIt.get();
      if (chamber->status)  
	if (hexist(80000+chamber->id)){
	  hreset(90000+chamber->id,"");
	  hopera(80000+chamber->id,"+",80000+chamber->id,90000+chamber->id,(float)1/event,0); 
	}
    }
    
    if ( ITR_rhp_iol.debug_mode.value )
      aframe_mess(routine, GRR_SINFO,
		  "Publishing Histograms, event = %d", event);
    
    /* get list of hist id's with first event */
    if ( event == 0) {
      rhp_hid(0, idhist, MAXHIST, &nhist);
      aframe_mess(routine,GRR_SINFO,
		  "Histogram list for publishing contains %d Ids",nhist);
    }
    if ( ITR_rhp_iol.debug_mode.value )
      aframe_mess(routine, GRR_SINFO,
		  "Publishing Histograms, event = %d", event);
    for (i=0; i<nhist; i++) {
      rc = rhp_publish(idhist[i]);
      if ( rc != GRR_OK )
	aframe_mess(routine,GRR_SERROR,"Error publishing Histo %d", idhist[i]);
    }
	

 
  }/*end of publishing*/


  /* count events */
  event++;
  
  if ( ITR_rhp_iol.debug_mode.value )
    aframe_mess(routine,GRR_SINFO,"Leaving RHP event...");
  
  return GRR_OK;
}

/*-------------------------------------------------------------
 * 
 * Store histogram file
 *
 *-------------------------------------------------------------*/
void ITR_rhp_storehisto(int run)
{
  int    i,hids=0;
  char   histfile[256];

  sprintf(histfile, "%s/dmon_itr_%d.his",
	  ITR_rhp_iol.path_histout.value, run);
  hrput(0, histfile, "n");
  
  hreset(hids,"");
}

/*-------------------------------------------------------------
 *
 * Terminate marvin online monitoring with rhp
 *
 *------------------------------------------------------------*/
int ITR_rhp_end()
{
  ITR_Silraw_Event_t *silraw;
  char *routine = "ITR_rhp_end";
  char histfile[256];
  /*int i,content[NSPACE];*/
  /* LedaTableIterator( ITR_Chamber, chamberIt );*/

  silraw = ITR_silraw_fetch_event();
  
  if ( !ITR_rhp_iol.use_rhp.value )
    return GRR_OK;

  /* store histogram file */
  sprintf(histfile, "%s/dmon_itr_%d.his",
	  ITR_rhp_iol.path_histout.value,silraw->header.major);
  if ( strcmp (ITR_rhp_iol.path_histout.value, "none") && 
      event >  ITR_rhp_iol.histevt_store.value)
    hrput(0,histfile,"n");

  /* book chamber-dependent histograms by looping over Leda chambers */
  /* for( chamberIt.begin(); chamberIt.valid(); chamberIt.next() ) {
    LedaTableRow(ITR_Chamber) *chamber = chamberIt.get();  
    printf("klappt\n"); 
    if (chamber->status){  
      printf("chamber: %x\n",chamber->id);  
      hunpak( 80000+chamber->id,*content," ",0);  
      for(i=0;i<NSTRIP;i++)  
	printf("strip i=%3d  content=%5d \t",i,content[i]);  
  } */ 

  /* terminate rhp */
  aframe_mess(routine,GRR_SINFO,"RHP terminated");

  return GRR_OK;
}


