#include "cif.h"
#include "rpm.h"
#include "rpi.h"
#include <string.h>
#include "grr.h"
#include <unistd.h>
#include <stdlib.h>
#include "cna.h"
#include "pub.h"
#include <sys/time.h>
#include <sys/types.h>

static char rpm_msg_data[RPM_LMAX];
static rpm_hdr rpm_msg_hdr;
static int run = 1;
static int ftab_id = CNA_MSG_MAX | (CNA_CLASS<<16);

static int handle_rpm(){
  int rc;
  if ( (rc=rpm_receive(&rpm_msg_hdr,&rpm_msg_data,RPM_LMAX,NO_BLOCK)) ) return rc;

  rc = rpm_dispatch(&rpm_msg_hdr,&rpm_msg_data);
  if ( rc && rc != GRR_UNKNOWN){
    printf("error return from rpm_dispatch %d\n",rc);
    return rc;
  }
  else if ( !rc ) return rc;

  switch(rpm_msg_hdr.type){
  case CNA_MSG_FTAB_NEEDED:
    {
      CNA_msg_ftab_needed *fn=(CNA_msg_ftab_needed *)rpm_msg_data;
      CNA_msg_ftab_needed_reply rep;
      char formatter[256];
      char *col;
      col = strchr(&(fn->ftab[0]),':');

      if ( !col ){
	printf("invalid formatted table '%s'\n",&(fn->ftab[0]));
	rep.rc = GRR_INVALID;
      }
      else{
	rpm_addr fmt;
	strncpy(formatter,&(fn->ftab[0]),col-&(fn->ftab[0]));
	formatter[col-&(fn->ftab[0])] = '\0';
	rc = rpm_locate(formatter,&fmt);
	if ( rc ){
	  printf("lookup of formatter '%s' returned %d\n",formatter,rc);
	  rep.rc = GRR_UNKNOWN;
	}
	else{
	  int len = sizeof(CNA_msg_sub_rtabs)+strlen(&(fn->ftab[0]))+1;
	  CNA_msg_sub_rtabs *sr=(CNA_msg_sub_rtabs *)malloc(len);
	  
	  sr->id = ftab_id++;
	  sr->dest = fn->dest;
	  strcpy(&(sr->ftab[0]),&(fn->ftab[0]));
	  rep.rc = rc = rpm_send(&fmt,sr,len,CNA_MSG_SUB_RTABS);
	  if ( rc )
	    printf("request for rtabs of ftab '%s' to '%s' returned %d\n",fn->ftab,formatter,rc);
	  else
	    printf("sent request for rtabs of ftab '%s' to '%s'\n",fn->ftab,formatter);
	  free(sr);
	}
	rc = rpm_reply(&rpm_msg_hdr,&rep,sizeof(rep),CNA_MSG_FTAB_NEEDED_REPLY);
	if ( rc )
	  printf("reply to client returned error %d\n",rc);
      }
    }
    break;
  case CNA_MSG_SUB_RTABS_REPLY:
    {
      CNA_msg_sub_rtabs_reply *rep=(CNA_msg_sub_rtabs_reply *)rpm_msg_data;
      if ( rep->rc )
	printf("request for rtabs replied with %d (%s)\n",rep->rc, grr_stript(rep->rc));
      else
	printf("successfull reply for rtabs\n");
    }
    break;
  case CNA_MSG_FTAB_READY:
    {
      CNA_msg_ftab_ready *req = (CNA_msg_ftab_ready*)rpm_msg_data;
      CNA_msg_ftab_ready_reply rep;
      rep.rc = GRR_OK;
      rc = rpm_reply(&rpm_msg_hdr,&rep,sizeof(rep),CNA_MSG_FTAB_READY_REPLY);
      if ( rc )
	printf("rpm_reply of kind CNA_MSG_FTAB_READY returned %d\n",rc);
      else
	printf("replied to ftab ready message for key %d\n",req->key);
    }
    break;
  case CNA_MSG_NEW_VERSION_REPLY:
    {
      CNA_msg_new_version_reply *rep = (CNA_msg_new_version_reply*)rpm_msg_data;
      printf("acknowledgement for new version with rc=%d\n",rep->rc);
    }
    break;
  case CNA_MSG_END_UPDATES_REPLY:
    {
      CNA_msg_end_updates_reply *rep = (CNA_msg_end_updates_reply*)rpm_msg_data;
      printf("acknowledgement for end-of-updates with rc=%d\n",rep->rc);
    }
    break;
  case CNA_MSG_FTAB_DIST_REPLY:
    {
      CNA_msg_ftab_dist_reply *rep = (CNA_msg_ftab_dist_reply*)rpm_msg_data;
      printf("acknowledgement for dist. ftabs with rc=%d\n",rep->rc);
    }
    break;
  case CNA_MSG_BEGIN_DIST_REPLY:
    {
      CNA_msg_begin_dist_reply *rep = (CNA_msg_begin_dist_reply*)rpm_msg_data;
      printf("acknowledgement for begin dist. with rc=%d\n",rep->rc);
    }
    break;
  case CNA_MSG_END_DIST_REPLY:
    {
      CNA_msg_end_dist_reply *rep = (CNA_msg_end_dist_reply*)rpm_msg_data;
      printf("acknowledgement for end dist. with rc=%d\n",rep->rc);
    }
    break;
  default:
    printf("received msg of type 0x%x\n",rpm_msg_hdr.type);
  }
  return 0;
}

static int handle_kb(){
  char inp[256];
  int rc = 0;
  if ( !fgets(inp,256,stdin) ) return 1;


  switch(*inp){
  case 'Q': 
  case 'q': 
    run = 0; 
    break;
  case 'b':
  case 'B':
    {
      char *line = inp+1;
      int len = strlen(line);
      if (!len) printf("raw table name required\n");
      else{
	if ( *line == ' ' ) {line++;len--;}
	if ( line[len-1] == '\n' ) len--;
	line[len] = '\0';
	rc = pub_book(line);
	if ( rc )
	  printf("pub_book for '%s' returned %d\n",line,rc);
	else
	  printf("booked new raw table '%s'\n",line);
	  
      }
    }
    break;
  case '\n':
    break;
  default:
    printf("undefined command %c (0x%x)\n",*inp,*inp);
  case 'U':
  case 'u':
    {
      char *line = inp+1;
      int len = strlen(line);
      if (!len) printf("formatter and raw table name required\n");
      else{
	CNA_msg_new_version nv;
	char name[200];
	sscanf(line,"%s%d%d",name,&nv.version[0],&nv.version[1]);
	rc = pub_publish(name,&nv,sizeof(nv),CNA_MSG_NEW_VERSION,-1);
	if ( rc )
	  printf("pub_publish for '%s' returned %d\n",name,rc);
	else
	  printf("updated raw table '%s' with versions %d/%d\n",
		 name,nv.version[0],nv.version[1]);
      }
    }
    break;
  case 'G':
  case 'g':
    {
      CNA_msg_end_updates eu;
      char fmt[200];
      rpm_addr dest;
      sscanf(inp+1,"%s%d",fmt,&eu.key);
      rc = rpm_locate(fmt,&dest);
      if ( rc ){
	printf("rpm_locate for '%s' returned %d\n",fmt,rc);
	break;
      }
      rc = rpm_send(&dest,&eu,sizeof(eu),CNA_MSG_END_UPDATES);
      if ( rc )
	printf("rpm_send for fmt '%s' key %d  complains with %d\n",fmt,eu.key,rc);
      else
	printf("send end-of-updates to fmt '%s' with key %d\n",fmt,eu.key);
    }
    break;
  case 'D':
  case 'd':
    {
      CNA_msg_ftab_dist di;
      char fmt[200];
      rpm_addr dest;
      sscanf(inp+1,"%s%d",fmt,&di.key);

      rc = rpm_locate(fmt,&dest);
      if ( rc ){
	printf("rpm_locate for '%s' returned %d\n",fmt,rc);
	break;
      }
      rc = rpm_send(&dest,&di,sizeof(di),CNA_MSG_FTAB_DIST);
      if ( rc )
	printf("rpm_send for fmt '%s' key %d  complains with %d\n",fmt,di.key,rc);
      else
	printf("send dist. ftabs to fmt '%s' with key %d\n",fmt,di.key);
    }
    break;
  case 'S':
  case 's':
    {
      CNA_msg_begin_dist bd;
      char cli[200];
      rpm_addr dest;
      sscanf(inp+1,"%s%d",cli,&bd.key);
      rc = rpm_locate(cli,&dest);
      if ( rc ){
	printf("rpm_locate for '%s' returned %d\n",cli,rc);
	break;
      }
      rc = rpm_send(&dest,&bd,sizeof(bd),CNA_MSG_BEGIN_DIST);
      if ( rc )
	printf("rpm_send to client '%s' key %d  complains with %d\n",cli,bd.key,rc);
      else
	printf("send begin dist. to client '%s' with key %d\n",cli,bd.key);
    }
    break;
  case 'E':
  case 'e':
    {
      CNA_msg_end_dist ed;
      char cli[200];
      rpm_addr dest;
      sscanf(inp+1,"%s%d",cli,&ed.key);
      rc = rpm_locate(cli,&dest);
      if ( rc ){
	printf("rpm_locate for '%s' returned %d\n",cli,rc);
	break;
      }
      rc = rpm_send(&dest,&ed,sizeof(ed),CNA_MSG_END_DIST);
      if ( rc )
	printf("rpm_send to client '%s' key %d  complains with %d\n",cli,ed.key,rc);
      else
	printf("send end dist. to client '%s' with key %d\n",cli,ed.key);
    }
    break;
  case 'h':
  case 'H':
    printf("list of commands:\n");
    printf("\t\033[1mq\033[0m\t\t\texit program\n");
    printf("\t\033[1mh\033[0m\t\t\tprint this list\n");
    printf("\t\033[1mb\033[0m raw_tab\t\tbook raw table\n");
    printf("\t\033[1mu\033[0m raw_tab major minor\tupdate raw table\n");
    printf("\t\033[1mg\033[0m formatter key \tgenerate formatted tabs\n");
    printf("\t\033[1md\033[0m formatter key \tdistribute formatted tabs\n");
    printf("\t\033[1ms\033[0m client key \t\tstart of distribution of ftabs\n");
    printf("\t\033[1me\033[0m client key \t\tend of distribution of ftabs\n");
  }

  return rc;
}

int main(int argc,const char **argv){
  int i;
  int rc;
  fd_set all_fds;
  int max_fd;

  rpm_init(argc,argv);
  rpm_open(RPM_REGISTER);

  grr_exec(argv[0]);
  run = 1;

  rc = pub_sinit();
  if ( rc ){
    printf("%s: pub_sinit complained with %d\n",argv[0],rc);
    run = 0;
  }
  else{
    rc = pub_sbind();
    if ( rc ){
      printf("%s: pub_sbind complained with %d\n",argv[0],rc);
      run = 0;
    }
  }    

  if ( (rc=cna_msgdef()) ){
    printf ("%s: cna_msgdef returned %d\n",argv[0],rc);
    run = 0;
  }

  FD_ZERO(&all_fds);
  rc = rpi_get_fds(-1,&all_fds); /* wait for all rpi/rpm channels */
  if ( rc ){
    printf("%s:rpi_get_fds returned %d\n",argv[0],rc);
    run = 0;
  }
  FD_SET(STDIN_FILENO, &all_fds);  /* wait for stdin */
  for(i=0;i<128;i++)
    if (FD_ISSET(i,&all_fds)) max_fd = i+1;


  rc = pub_book("/DUMMY:dummy");
  if ( rc )
    printf("pub_book for '/DUMMY:dummy' returned %d\n",rc);
  else
    printf("booked new raw table '/DUMMY:dummy'\n");



  while(run){
    fd_set rfds;
    struct timeval tv;
    int stat = 0;
    rfds = all_fds;
    /* Wait up to five seconds. */
    tv.tv_sec = 5;
    tv.tv_usec = 0;
    stat = select(max_fd, &rfds, NULL, NULL, &tv);
    if ( stat<0 )
      printf("select returned %d\n",stat);

    /* try to read from keyboard only if data availlable */
    if ( FD_ISSET(STDIN_FILENO,&rfds) )
      handle_kb();
    /* rpm is save since it is non-blocking */
    handle_rpm();
  }

  rpm_halt();

  return 0;
}

