#include "itr_cif.h"

#include "itrleda.h"
#include "itrleda_structs.h"

# define N_RTABS 9

static char *tablename[N_RTABS] = {
    "/ITR_Setup:ITR/Chamber",
    "/ITR_Setup:ITR/nesl",
    "/ITR_Setup:ITR/nede",
    "/ITR_Setup:ITR/Fed", 
    "/ITR_Setup:ITR/Chip",
    "/ITR_MONITOR:ITR/Chanstat",
    "/ITR_MONITOR:ITR/Ped",
    "/ITR_MONITOR:ITR/Noise",
    "/ITR_Align:ITR/Align"
};

static char *tableleda[N_RTABS] = {
    "ITR_Chamber",
    "ITR_nesl",
    "ITR_nede",
    "ITR_Fed",  
    "ITR_Chip", 
    "ITR_Chanstat", 
    "ITR_Ped", 
    "ITR_Noise",
    "ITR_Align" 
};

static int ftab_id;
static char *myftab = "itr_constants";
static int itr_rawtabid[N_RTABS];

itrleda_fmtTable_t itr_tables;

int itr_fmtr_subsc (const char *tab, int id, CIF_format_tab ** formatter)
{
    int rc = 0;
    int rtab;

    for (rtab = 0; rtab < N_RTABS; rtab++) {

	rc = CIF_formatter_need_raw (id, tablename[rtab], &itr_rawtabid[rtab]);
	if (rc != GRR_OK) {
	    printf ("Error while subscribing to raw table %s: %d (%s)\n",
		    tablename[rtab], rc, grr_stript (rc));
	    return rc;
	}
    }

    ftab_id = id;
    *formatter = &itr_constant_hdl;

    return rc;
}




int itr_formatter_setup (int argc, const char **argv)
{
    char rpm_itr_const[2000]; /* WARNING:  for RPM to work, strlen(rpm_itr_const) MUST be < 1000 */
    char rpm_preheader[300], rpm_chamber_table[300];
    char rpm_fed_table[300], rpm_chip_table[300];
    char rpm_align_table[300], rpm_chanstat[300];
    char rpm_noise_table[300], rpm_ped_table[300];
    char rpm_nede_table[600], rpm_nesl_table[600];
    char *routine = "itr_formatter_setup";
    int rc = 0;

    sprintf (rpm_preheader, "major:I,minor:I,date:200B,number:I");

    sprintf (rpm_chamber_table,
	     "nChambers:I,"
	     "Chamber:%d("
	     "id:US,"
	     "name:50B,"
	     "align_flag:B,"
	     "status:B," "type:UB," "chip:6US," "align:US," "tonede:US)", MAX_FMT_CHAMBERS);

    sprintf (rpm_fed_table,
	     "nFeds:I,"
	     "Fed:%d("
	     "id:US,vmeid:UB,board:UB,sharc:UB,link:UB,chip:2US)",
	     MAX_FMT_FEDS);
    

    sprintf (rpm_chip_table,
	     "nChips:I,"
	     "Chip:%d("
	     "id:US,"
	     "ncellped:UB,"
	     "ncellnoise:UB," "status:B," "pipelen:UB," "fed:US," "chamber:US)", MAX_FMT_CHIPS);

    sprintf (rpm_chanstat,
	     "nChanstat:I,Chanstat:%d(chipid:US,status:%dUB)", MAX_FMT_CHIPS, NCHIPCHAN);

    sprintf (rpm_ped_table, "nPed:I,Ped:%d(chipid:US,pedestal:%dS)", MAX_FMT_CHIPS, NCHIPCHAN);

    sprintf (rpm_noise_table,
	     "nNoise:I,Noise:%d(chipid:US,rawvar:%dF,subvar:%dF)",
	     MAX_FMT_CHIPS, NCHIPCHAN, NCHIPCHAN);

    sprintf (rpm_align_table,
	     "nAlign:I,"
	     "Align:%d(id:US,nede:I,ang:3F,eang:3F,delt:3F,"
	     "edelt:3F,sloff:3F,esloff:3F,redres:F)", MAX_FMT_ALIGN);

    sprintf (rpm_nesl_table,
	     "nNesl:I,"
	     "Nesl:%d("
	     "inesl:I,nsl:I,mfl:I,lay:I,xf:F,xl:F,yf:F,yl:F,zf:F,zl:F,dx:F,dy:F,"
	     "npl:I,fpl:I,nwl:I,fwl:I,ltyp:I,pnede:I,name:5B,tonede:I)", MAX_FMT_NESL);

    sprintf (rpm_nede_table,
	     "nNede:I,"
	     "Nede:%d("
	     "inede:I,rot_db:9F,xfw_db:2F,zfw_db:F,nwi_db:2I,xmin_db:2F,xmax_db:2F,"
	     "ymin_db:2F,ymax_db:2F,zmin_db:F,zmax_db:F,sxmn_db:2F,sxmx_db:2F,"
	     "symx_db:2F,symn_db:2F,acc_db:F,eff_db:F,dod_db:F,pit_db:F,ms_db:F,"
	     "nsl_db:I,superl_db:I,lay_db:I,sec_db:2I,dedx_db:F,hwc_db:13B,"
	     "noise_db:I,ares_db:I,tonesl:I,tochamber:US,toalign:US)", MAX_FMT_NEDE);
	     

    strcpy (rpm_itr_const, rpm_preheader);
    strcat (rpm_itr_const, ",");
    strcat (rpm_itr_const, rpm_chamber_table);
    strcat (rpm_itr_const, ",");
    strcat (rpm_itr_const, rpm_fed_table); 
    strcat (rpm_itr_const, ","); 
    strcat (rpm_itr_const, rpm_chip_table);
    strcat (rpm_itr_const, ",");
    strcat (rpm_itr_const, rpm_chanstat);
    strcat (rpm_itr_const, ",");
    strcat (rpm_itr_const, rpm_ped_table);
    strcat (rpm_itr_const, ",");
    strcat (rpm_itr_const, rpm_noise_table);
    strcat (rpm_itr_const, ",");
    strcat (rpm_itr_const, rpm_align_table);
    strcat (rpm_itr_const, ",");
    strcat (rpm_itr_const, rpm_nesl_table);
    strcat (rpm_itr_const, ",");
    strcat (rpm_itr_const, rpm_nede_table);

    /* printf ("RPM Format being used: \n%s\n (len: %d) \n", 
       rpm_itr_const, strlen(rpm_itr_const));                   */

    rc = CIF_add_formatted_tab (myftab, rpm_itr_const, &itr_fmtr_subsc);
    if (rc != GRR_OK)
	return rc;


    /* ---------------------------------------------------   
     * Initialization
     * --------------------------------------------------- */

    /* Initilialize some control words */
    aframe_store_program_name (argv[0]);

    /* initialize parameters */
    aframe_param_init ();
    iol_param_init ();
    /* -------------------------------------------------------------
     * start up the Leda stuff 
     * ------------------------------------------------------------ */
    ITR_leda_param_init ();
    ITR_leda_initialize (itrleda_CheckDD_ALWAYS);

    /* Name of the Database now hardcoded for the vdsfmt. GS 24.5.00 */
    if (strcmp (itrleda_iol.setup_database.value, "/ITR_Setup")) {
	aframe_mess (routine, GRR_SFATAL,
		     "ITR database not properly specified! Check iol settings.");
	return GRR_FISHY;
    }

    /* preferences are loaded here */
    if (aframe_process_args (argc, (const char **) argv, AFRAME_CLI) != AFRAME_OK) {
	aframe_mess (routine, GRR_SFATAL, "Error processing arguments.");
	return GRR_FISHY;
    }


    /* fetch initial revision of database tables */
    ITR_leda_startprogram (0, 0);

    return rc;
}




int itr_constant_hdl (const CIF_raw_tab * updated_rtabs, int N_raw,
		      CIF_formatted_tab * updated_ftabs, int N_max, int *N_updated)
{

    char *routine = "itr_constant_hdl";
    int j, k, major = 0, minor = 0,
      nCham = 0, nChip = 0, nChanstat = 0, nPed = 0, nNoise = 0, nAlign = 0, 
      nNesl = 0, nNede = 0, nFed = 0,
      rc = GRR_OK;
    extern itrleda_fmtTable_t itr_tables;

    /* -------------------------------------------------------------
     * read in all the DB contents which have been updated 
     * and fill corresponding parts of itr_tables structure 
     * -------------------------------------------------------------- */

    /* Check that the size of the array for the formatted tables is large enough */

    *N_updated = 0;
    if (N_max < 1)
	return GRR_FISHY;

    /* Loop over the updated raw tables, to identify and get them from the DB,
     * and fill their contents in the corresponding parts of the ftab itr_tables */

    printf (">>>>>>>> Number of raw tables:%d\n", N_raw);
    for (j = 0; j < N_raw; j++) {
	printf (">>>>>>>> id:%d, major:%d, minor:%d\n",
		updated_rtabs[j].id, updated_rtabs[j].version[0], updated_rtabs[j].version[1]);
    }

    if (N_raw > 0) {
	for (j = 0; j < N_raw; j++) {	/* loop over the updated rtabs */
	    for (k = 0; k < N_RTABS; k++) {	/* loop over all rtabs to identify the ID */
		if (updated_rtabs[j].id == itr_rawtabid[k]) {
		    printf ("tablename:%s, myid:%d, major:%d, minor:%d\n",
			    tableleda[k], itr_rawtabid[k],
			    updated_rtabs[j].version[0], updated_rtabs[j].version[1]);

		    if ((rc = itrleda_fetchtable(tableleda[k],
						 updated_rtabs[j].version[0],
						 updated_rtabs[j].version[1])) != GRR_OK) {
			aframe_mess(routine,GRR_SFATAL,"Failed to open Database or Table.");
			return GRR_FISHY;
		    }

		    /*  k = N_RTABS; */
		}
	    }
	}
    }

    ITR_leda_Chamber_structFromTable (itr_tables.chambers, MAX_FMT_CHAMBERS, &nCham);
    itr_tables.nChambers = nCham;

    ITR_leda_Fed_structFromTable (itr_tables.fed, MAX_FMT_FEDS, &nFed);
    itr_tables.nFeds = nFed;

    ITR_leda_Chip_structFromTable (itr_tables.chips, MAX_FMT_CHIPS, &nChip);
    itr_tables.nChips = nChip;

    ITR_leda_Chanstat_structFromTable (itr_tables.chanstat, MAX_FMT_CHIPS, &nChanstat);
    if (nChanstat != itr_tables.nChips) {
	printf (" *** warning:  Chanstat table " 
		"and Chip table have different sizes!\n");
    }
    itr_tables.nChanstat = nChanstat;


    ITR_leda_Ped_structFromTable (itr_tables.ped, MAX_FMT_CHIPS, &nPed);
    if (nPed != itr_tables.nChips) {
	printf (" *** warning:  Ped table " 
		"and Chip table have different sizes!\n"); 
    }
    itr_tables.nPed = nPed;

    ITR_leda_Noise_structFromTable (itr_tables.noise, MAX_FMT_CHIPS, &nNoise);
    if (nNoise != itr_tables.nChips) {
	printf (" *** warning:  Noise table " 
		"and Chip table have different sizes!\n");
    }
    itr_tables.nNoise = nNoise;

    ITR_leda_Align_structFromTable (itr_tables.align, MAX_FMT_ALIGN, &nAlign);
    itr_tables.nAlign = nAlign;

    ITR_leda_nesl_structFromTable  (itr_tables.nesl,  MAX_FMT_NESL, &nNesl);
    itr_tables.nNesl  = nNesl;

    ITR_leda_nede_structFromTable  (itr_tables.nede,  MAX_FMT_NEDE, &nNede);
    itr_tables.nNede  = nNede;

    printf (" The number of requested  updates were %d\n", N_raw);
    printf (" The number of fetched updates were %d\n", j);

    strncpy (itr_tables.date, print_date (time (NULL)), 199);

    /* tell CIF where to find the structure with the new formatted table */
    updated_ftabs[0].data = &itr_tables;
    updated_ftabs[0].id = ftab_id;
    updated_ftabs[0].len = sizeof itr_tables;
    *N_updated = 1;
    printf ("size of ftab:%d\n", updated_ftabs[0].len);

    return rc;
}



/*
 * Local variables:
 *  c-basic-offset: 4
 * End:
 */
