/* ---------------------------------------------------------------
 *
 *  fill channel map list with values stored in database
 *  and retrieved by LEDA
 *
 *  Returned is the pointer to the first channel map
 *
 *  Function argument:
 *     int *nmap;    // number of maps
 *
 * --------------------------------------------------------------- */

#include <iostream.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "component.h"
#include "grr.h"

#include "aframelib.h"

#include "ledaC++.hh"
#include "itrleda.h"
#include "itrleda_dd.h"
#include "itrMAP.h"

#define NCHIPCHAN      128
#define MAXMAP         800
#define MAXFEDCHAN    (2*NCHIPCHAN)

ITR_map* itrslt_fillMAP( int* nmap )
{
  char *routine = "itrslt_fillMAP";

  static ITR_map* itrmap = NULL;  /* pointer to MAP */

  /* define a LEDA table iterator for looping over FEDs */
  LedaTableIterator<class ITR_Chamber> chamberIt;

  /* initialize map counter */
  *nmap = 0;

  /* allocate space for MAP table */
  if ( itrmap == NULL ) {
    itrmap = (ITR_map*) calloc( sizeof(ITR_map), MAXMAP );
    if ( itrmap == NULL ) {
      aframe_mess(routine,GRR_SERROR,
		  "Could not allocate space for ITR_map table");
      return NULL;
    }
  }

  const int kOffset      = 8;
  const int kBitsInColNo = 8;

  int cnt    = 0;
  int oldFED = -1, chipNo;
  /* loop over Chamber entries in LEDA */
  for (  ; chamberIt.valid(); chamberIt.next() ) {
    LedaPointer<class ITR_Chamber>  chamber = chamberIt.get();
    LedaRelationIterator chipIt (chamber->chip);
    LedaPointer<class ITR_nede> nede = chamber->tonede;
    LedaPointer<class ITR_nesl> nesl = nede->tonesl;

    int nWi = 0;
    int wireoffset;

    if (chamber->type == 3)
      wireoffset = 5; 
    else
      wireoffset = 8;

    for (chipNo = 0 ; chipIt.valid() && cnt < MAXMAP; chipIt.next(), chipNo++) {
      LedaPointer<class ITR_Chip>   chip (chipIt);
      LedaPointer<class ITR_Fed>    fed    = chip->fed;

      LedaTableIterator <class ITR_FedSlb> fedslb_it;
      LedaPointer<class ITR_FedSlb> fedslb;
      int slbfound = 0;
      for (; fedslb_it.valid(); fedslb_it.next()) {
	fedslb = fedslb_it.get();
	if (fedslb->fedid == fed->id) { 
	  slbfound = 1;
	  break;
	}
      }
      
      if (fed->id != oldFED) {                // we haven't seen this before
	if (cnt >= MAXMAP) {
	  cout << "too many maps! " << endl;
	  break;
	}
	
	oldFED = fed->id;

	strncpy(itrmap[cnt].spl, nesl->name, 5);
	itrmap[cnt].cid    = COMP_N_ITR;
	itrmap[cnt].sl     = nede->superl_db;
	itrmap[cnt].lay    = nede->lay_db;
	itrmap[cnt].sec    = nede->sec_db[0];
	itrmap[cnt].fedid  = fed ->id;
	itrmap[cnt].vmeid  = fed ->vmeid;
	itrmap[cnt].board  = fed ->board;
	itrmap[cnt].sharc  = fed ->sharc;
	itrmap[cnt].link   = 0;              /* DAQ ID to be filled in here! */
	itrmap[cnt].daqid  = (slbfound) ? fedslb->slbid : -1;

	if (chipNo == 0) { // first chip:  some channels are not bonded to chamber
	  itrmap[cnt].chan1  = kOffset + wireoffset; /* this offset has to come */
						     /*	from DB! */
	  itrmap[cnt].chan1a = kOffset + NCHIPCHAN;
	  
	  itrmap[cnt].wire1  = 0;
	  itrmap[cnt].wire1a = NCHIPCHAN - wireoffset - 1;
	  nWi = NCHIPCHAN - wireoffset;
	}
	else { 
	  itrmap[cnt].chan1  = kOffset; /* this offset has to come */
						     /*	from DB! */
	  itrmap[cnt].chan1a = kOffset + NCHIPCHAN - 1;
	  
	  itrmap[cnt].wire1  = nWi;
	  itrmap[cnt].wire1a = nWi + NCHIPCHAN - 1;
	  nWi += NCHIPCHAN;
	}
      }
      else {
	if (chipNo == 5) { // last chip:  some channels are not bonded to chamber
	  itrmap[cnt].chan1b = kOffset + NCHIPCHAN + kBitsInColNo; /* this offset has to come */
						                   /* from DB! */
	  itrmap[cnt].chan2  = itrmap[cnt].chan1b + NCHIPCHAN - wireoffset - 1;
	  
	  itrmap[cnt].wire1b  = nWi;
	  itrmap[cnt].wire2   = nWi + NCHIPCHAN - wireoffset - 1;
	  nWi += NCHIPCHAN - wireoffset;
	}
	else { 
	  itrmap[cnt].chan1b = kOffset + NCHIPCHAN + kBitsInColNo; /* this offset has to come */
	                                                           /*	from DB! */
	  itrmap[cnt].chan2  = itrmap[cnt].chan1b + NCHIPCHAN - 1;
	  
	  itrmap[cnt].wire1b = nWi;
	  itrmap[cnt].wire2  = nWi + NCHIPCHAN - 1;
	  nWi += NCHIPCHAN;
	}

	cnt ++;
      }
    } /* end loop over Chips */
  }  /* end loop over Chambers */
  
  *nmap = cnt;
  return itrmap;
}

